clear all;

% INPUT PARAMETERS
distance=50;         % m
bandwidth=0.08;          % GHz
center_freq=0.75;       % GHz
time_period=4000;       % ns
channel=2;              % 1: DIAMOND REPUBLIC PLAZA
                        % 2: SUNCOR OIL REFINERY
                        % 3: NIST BOULDER LAB
                        % 4: HORIZONS WEST APARTMENTS
                        % 5: DENVER CONVENTION CENTER
                        % 6: GREATHOUSE TUNNEL
                        % 7: HAZEL-ATLAS TUNNEL
                   
% CHANNEL PARAMETERS
switch(channel)

% DIAMOND REPUBLIC PLAZA
case 1
    lambda=37.4002; kappa=4.01546; 
    Lambda_plus=97.0719; Kappa_plus=3.40003;
    Lambda=582.973; Kappa=1.49177;

    gamma0_plus=-7.405; gamma1_plus=-0.1166; gamma2_plus=4.26;
    sigma_gamma_plus=0.0986962;
    gamma0=-6.008; gamma1=-1.123; gamma2=-0.01779;
    sigma_gamma=0.0106059;

    N=0.75;   
    Gamma0=1213; Gamma1=-0.6545; Gamma2=-84.39;
    sigma_Gamma=7.78883;

    sigma_power=3.38185;

    n0=-5.952; PL0=57.17; sigma=3.01503;
    d1=inf; n1=0; PL1=0;

% SUNCOR OIL REFINERY
case 2
    lambda=54.0368; kappa=2.9997; 
    Lambda_plus=0; Kappa_plus=1;
    Lambda=883.936; Kappa=1.57143;

    gamma0_plus=0; gamma1_plus=0; gamma2_plus=1;
    sigma_gamma_plus=0;
    gamma0=-492.7; gamma1=-1.615; gamma2=-0.004604;
    sigma_gamma=0.0328705;

    N=2.5;   
    Gamma0=553.6; Gamma1=-0.3664; Gamma2=-115.4;
    sigma_Gamma=6.34551;

    sigma_power=2.79382;

    n0=-0.3468; PL0=-17.9; sigma=1.937;
    d1=87; n1=-6.618; PL1=84.22;
    
% NIST BOULDER LAB
case 3
    lambda=42.7323; kappa=3.70698; 
    Lambda_plus=90.6667; Kappa_plus=2.42727;
    Lambda=396.343; Kappa=1.89343;

    gamma0_plus=1.371e4; gamma1_plus=-1.941; gamma2_plus=-0.0189;
    sigma_gamma_plus=2.76616;
    gamma0=-1.492e4; gamma1=-2.309; gamma2=-0.01915;
    sigma_gamma=0.0143792;

    N=0.5714;   
    Gamma0=2498; Gamma1=-0.6993; Gamma2=-83.24;
    sigma_Gamma=3.80266;

    sigma_power=2.57634;

    n0=-4.331; PL0=52.76; sigma=3.17674;
    d1=inf; n1=0; PL1=0;

% HORIZONS WEST APARTMENTS
case 4
    lambda=42.3649; kappa=4.50559; 
    Lambda_plus=0; Kappa_plus=1;
    Lambda=565.695; Kappa=2.65972;

    gamma0_plus=0; gamma1_plus=0; gamma2_plus=1;
    sigma_gamma_plus=0;
    gamma0=-0.3608; gamma1=0.1096; gamma2=0.1446;
    sigma_gamma=0.02333659;

    N=1;   
    Gamma0=693.7; Gamma1=-0.04428; Gamma2=-563.8;
    sigma_Gamma=5.04885;

    sigma_power=3.00107;

    n0=-1.821; PL0=21.6591; sigma=4.8405;
    d1=inf; n1=0; PL1=0;

% DENVER CONVENTION CENTER
case 5
    lambda=35.7586; kappa=3.62971; 
    Lambda_plus=0; Kappa_plus=1;
    Lambda=591.052; Kappa=3.68634;

    gamma0_plus=0; gamma1_plus=0; gamma2_plus=1;
    sigma_gamma_plus=0;
    gamma0=-0.3032; gamma1=-0.3932; gamma2=0.002615;
    sigma_gamma=0.0099432;

    N=0.5;   
    Gamma0=3399; Gamma1=-0.003547; Gamma2=-3365;
    sigma_Gamma=4.14005;

    sigma_power=3.38158;

    n0=-7.257; PL0=118.2; sigma=5.12157;
    d1=inf; n1=0; PL1=0;

% GREATHOUSE TUNNEL
case 6
    lambda=38.0454; kappa=2.69736; 
    Lambda_plus=0; Kappa_plus=1;
    Lambda=154.626; Kappa=15.1727;

    gamma0_plus=0; gamma1_plus=0; gamma2_plus=1;
    sigma_gamma_plus=0;
    gamma0=-21.72; gamma1=-0.003732; gamma2=21.14;
    sigma_gamma=0.0970348;

    N=0.8182;   
    Gamma0=-0.008303; Gamma1=1.451; Gamma2=6.915;
    sigma_Gamma=13.493;

    sigma_power=1.55689;

    n0=-0.5514; PL0=18.4691; sigma=0.531682;
    d1=70; n1=-19.04; PL1=338.8691;

% HAZEL-ATLAS TUNNEL
case 7
    lambda=34.3376; kappa=3.2161; 
    Lambda_plus=0; Kappa_plus=1;
    Lambda=0; Kappa=1;

    gamma0_plus=0; gamma1_plus=0; gamma2_plus=1;
    sigma_gamma_plus=0;
    gamma0=-1.177; gamma1=-0.01202; gamma2=0.9546;
    sigma_gamma=0.0416636;

    N=0;   
    Gamma0=-8.548e-9; Gamma1=3.799; Gamma2=-20.62;
    sigma_Gamma=5.16379;

    sigma_power=3.45234;

    n0=-0.2579; PL0=-12.2309; sigma=2.49292;
    d1=60; n1=-21.19; PL1=413.1309;
    
otherwise
    display('INPUT PARAMETER ERROR: CHANNEL MUST BE INTEGER VALUE BETWEEN 1-7');
    break;
end



if(time_period<0)
    display('INPUT PARAMETER ERROR: TIME PERIOD MUST BE POSITIVE');
    break;
end
if(distance<0)
    display('INPUT PARAMETER ERROR: DISTANCE MUST BE POSITIVE');
    break;
end


% GENERATE POWER DELAY PROFILE
TOA=distance/0.3;   

dim_arrival=0; clear arrival;

% GENERATE CLUSTERS WITH NEGATIVE SLOPE
time_cluster=TOA;
time_cluster=time_cluster+wblrnd(Lambda_plus,Kappa_plus);

eta=poissrnd(N);
for i=1:1+eta
    time_arrival=time_cluster;
    power_cluster=normrnd(Gamma0*time_cluster^Gamma1+Gamma2,sigma_Gamma);
    if(i==1)
        power_cluster0=power_cluster;
        time_cluster0=time_cluster;
    end
    
    slope_cluster=0;
    while(slope_cluster>=0)
        slope_cluster=normrnd(gamma0*time_cluster^gamma1+gamma2,sigma_gamma);
    end
    
    while(time_arrival<time_period)    
        power_arrival=normrnd(power_cluster+slope_cluster*(time_arrival-time_cluster),sigma_power);
       
        dim_arrival=dim_arrival+1;
        arrival(dim_arrival).time=time_arrival;
        arrival(dim_arrival).power=power_arrival;

        time_arrival=time_arrival+wblrnd(lambda,kappa);
    end

    time_cluster=time_cluster+wblrnd(Lambda,Kappa);
end

% GENERATE CLUSTERS WITH POSITIVE SLOPE
time_cluster=TOA;

slope_cluster=0;
while(slope_cluster<=0)
    slope_cluster=normrnd(gamma0_plus*time_cluster^gamma1_plus+gamma2_plus,sigma_gamma_plus);
end
power_cluster=power_cluster0-slope_cluster*(time_cluster0-time_cluster);

time_arrival=time_cluster+wblrnd(lambda,kappa);
while(time_arrival<time_cluster0)
    power_arrival=normrnd(power_cluster+slope_cluster*(time_arrival-time_cluster),sigma_power);
     
    dim_arrival=dim_arrival+1;
    arrival(dim_arrival).time=time_arrival;
    arrival(dim_arrival).power=power_arrival;
  
    time_arrival=time_arrival+wblrnd(lambda,kappa);
end


% CONVERT POWER TO COMPLEX AMPLITUDE WHOSE PHASE IS A UNIFORM RANDOM
% VARIABLE
for k=1:dim_arrival
    arrival(k).amp=10^(arrival(k).power/20)*exp(j*2*pi*rand(1));
end

delta_freq=1/time_period;
low_freq=center_freq-bandwidth/2;
high_freq=center_freq+bandwidth/2;

if(low_freq<0.7 || high_freq>0.8 || bandwidth<0)
    display('INPUT PARAMETER ERROR: BAND MUST BE BETWEEN 0.7-0.8 GHz');
    break;
end

dim_H=0; clear H;
for freq=low_freq:delta_freq:high_freq
    dim_H=dim_H+1;
    H(dim_H)=0;
    for k=1:dim_arrival
        H(dim_H)=H(dim_H)+arrival(k).amp*exp(-j*2*pi*freq*arrival(k).time);
    end
end


% GENERATE FREQUENCY RESPONSE FOR SPECIFIED BAND
power_H=sum(H*H');
if(distance<=d1)
    pathloss=10^((normrnd(n0*10*log10(distance)+PL0,sigma))/10);
else
    pathloss=10^((normrnd(n1*10*log10(distance)+PL1,sigma))/10);
end
H=H*sqrt(pathloss/power_H);
index_H=low_freq:delta_freq:high_freq;
figure(1); clf; 
plot(index_H,abs(H));
axis tight;
title('Frequency response'); xlabel('f: frequency (GHz)'); ylabel('|H(f)|');


% GENERATE TIME RESPONSE FOR SPECIFIED BAND
alpha=2.5;
h=ifft(H.*gausswin(dim_H,alpha).',time_period);
power_h=sum(h*h');
h=h*sqrt(pathloss/power_h);
index_h=0:time_period-1;
figure(2); clf;
plot(index_h,abs(h));
axis tight;
title('Time response'); xlabel('t: time (ns)'); ylabel('|h(t)|');